unit DelphiGzip;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DsgnIntf, gzio;

type
  TZeroHundred = 0..100;

  TAboutProperty = class(TPropertyEditor)
  public
	procedure Edit; override;
	function GetAttributes: TPropertyAttributes; override;
	function GetValue: string; override;
  end;

  TCompressionLevel = 1..9;
  TCompressionType = (Standard,Filtered,HuffmanOnly);
  TGzip = class(TComponent)
  private
	{ Private declarations }
	FAbout : TAboutProperty;
	FFileSource : string;
	FFileDestination : string;
	FDeleteSource : boolean;
	FCompressionLevel : TCompressionLevel;
	FCompressionType : TCompressionType;
	FWindowOnError : boolean;
	FOnProgress : TNotifyEvent;
	FProgress : integer;
	FProgressStep : TZeroHundred;
  protected
	{ Protected declarations }
	procedure DoOnProgress; virtual;
	function gz_compress (var infile:file; outfile:gzFile): integer;
	function gz_uncompress (infile:gzFile; var outfile:file;
							fsize:longword) : integer;
  public
	{ Public declarations }
	constructor Create( AOwner: TComponent); override;
	procedure FileSwitch;
	function Gzip : integer;
	function Gunzip : integer;
	property Progress : integer
		 read FProgress write FProgress;
  published
	{ Published declarations }
	property About: TAboutProperty
		 read FAbout write FAbout;
	Property DeleteSource : boolean
		 read FDeleteSource write FDeleteSource;
	Property FileSource : string
		 read FFileSource write FFileSource;
	Property FileDestination : string
		 read FFileDestination write FFileDestination;
	Property CompressionLevel : TCompressionLevel
		 read FCompressionLevel write FCompressionLevel;
	Property CompressionType : TCompressionType
		 read FCompressionType write FCompressionType;
	Property WindowOnError : boolean
		 read FWindowOnError write FWindowOnError;
	property ProgressStep : TZeroHundred
		 read FProgressStep write FProgressStep;
	property OnProgress : TNotifyEvent
		 read FOnProgress write FOnProgress;
  end;

  procedure Register;

implementation

uses utils;

procedure TAboutProperty.Edit;
var utils : TUtils;
begin
   ShowMessage(utils.CreateAboutMsg('DelphiGZip'))
end;

function TAboutProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paDialog, paReadOnly];
end;

function TAboutProperty.GetValue: string;
begin
  Result := 'DelphiGzip';
end;

constructor TGzip.Create( AOwner: TComponent);
begin
   inherited Create( AOwner);
   CompressionLevel := 6;
   CompressionType := Standard;
   FileSource := 'data.dat';
   FileDestination := 'data.dat.gz';
   DeleteSource := False;
   WindowOnError := True;
   FProgressStep := 0
end;

procedure TGzip.DoOnProgress;
begin
	if Assigned (FOnProgress) then
	   FOnProgress (self)
end;

{ gz_compress ----------------------------------------------
# This code comes from minigzip.pas with some changes
# Original:
# minigzip.c -- usage example of the zlib compression library
# Copyright (C) 1995-1998 Jean-loup Gailly.
#
# Pascal tranlastion
# Copyright (C) 1998 by Jacques Nomssi Nzali
#
# 0 - No Error
# 1 - Read Error
# 2 - Write Error
# 3 - gzclose error
-----------------------------------------------------------}
function TGzip.gz_compress (var infile:file; outfile:gzFile): integer;
var
  len   : uInt;
  ioerr : integer;
  buf  : packed array [0..BUFLEN-1] of byte; { Global uses BSS instead of stack }
  errorcode : byte;
  fsize, lensize : longword;

begin
  errorcode := 0;
  Progress := 0;
  fsize := FileSize(infile);
  lensize := 0;
  if FProgressStep > 0 then DoOnProgress;

  while true do begin
	{$I-}
	blockread (infile, buf, BUFLEN, len);
	{$I+}

	ioerr := IOResult;
	if (ioerr <> 0) then begin
	  errorcode := 1;
	  break
	end;

	if (len = 0) then break;

	{$WARNINGS OFF}{Comparing signed and unsigned types}
	if (gzwrite (outfile, @buf, len) <> len) then begin
	{$WARNINGS OFF}
	  errorcode := 2;
	  break
	end;

	if FProgressStep > 0 then begin
	   {$WARNINGS OFF}
	   lensize := lensize + len;
	   if ((lensize / fsize) * 100 >= FProgress + FProgressStep)
						or (lensize = fsize) then begin
		  FProgress := Trunc((lensize / fsize) * 100);
		  DoOnProgress
	   end
	   {$WARNINGS ON}
	end
  end; {WHILE}

  closeFile (infile);
  if (gzclose (outfile) <> 0{Z_OK}) then errorcode := 3;

  gz_compress := errorcode;
end;

{ gz_uncompress ----------------------------------------------
# This code comes from minigzip.pas with some changes
# Original:
# minigzip.c -- usage example of the zlib compression library
# Copyright (C) 1995-1998 Jean-loup Gailly.
#
# Pascal tranlastion
# Copyright (C) 1998 by Jacques Nomssi Nzali
#
# 0 - No error
# 1 - Read Error
# 2 - Write Error
# 3 - gzclose Error
-----------------------------------------------------------}
function TGzip.gz_uncompress (infile:gzFile; var outfile:file;
							  fsize:longword) : integer;
var
  len     : integer;
  written : uInt;
  ioerr   : integer;
  buf  : packed array [0..BUFLEN-1] of byte; { Global uses BSS instead of stack }
  errorcode : byte;
  lensize : longword;
begin
  errorcode := 0;
  FProgress := 0;
  lensize := 0;
  if FProgressStep > 0 then DoOnProgress;

  while true do begin

	len := gzread (infile, @buf, BUFLEN);
	if (len < 0) then begin
	   errorcode := 1;
	   break
	end;
	if (len = 0)
	  then break;

	{$I-}
	blockwrite (outfile, buf, len, written);
	{$I+}
	{$WARNINGS OFF}{Comparing signed and unsigned types}
	if (written <> len) then begin
	{$WARNINGS ON}
	   errorcode := 2;
	   break
	end;

	if FProgressStep > 0 then begin
	   {$WARNINGS OFF}
	   lensize := lensize + len;
	   if ((lensize / fsize) * 100 >= FProgress + FProgressStep)
						or (lensize = fsize) then begin
		  FProgress := Trunc((lensize / fsize) * 100);
		  DoOnProgress
	   end
	   {$WARNINGS ON}
	end
  end; {WHILE}

  {$I-}
  close (outfile);
  {$I+}
  ioerr := IOResult;
  if (ioerr <> 0) then begin
	if FWindowOnError then
	   MessageDlg('Can''t close file.', mtError, [mbAbort], 0);
	halt(1)
  end;

  if (gzclose (infile) <> 0{Z_OK}) then begin
	 if FWindowOnError then
		MessageDlg('gzclose Error.', mtError, [mbAbort], 0);
	 errorcode := 3
  end;

  gz_uncompress := errorcode
end;

{***************************************************************
* The public part
***************************************************************}

procedure TGzip.FileSwitch;
var s : string;
begin
   s := FFileSource;
   FFileSource := FFileDestination;
   FFileDestination := s;
end;

{ Gzip ---------------------------------------------------------
# Returns 0 - File compressed
#         1 - Could not open FFileIn
#         2 - Could not create FFileOut
#      >100 - Error-100 in gz_compress
---------------------------------------------------------------}
function TGzip.Gzip : integer;
var outmode : string;
	s : string;
	infile  : file;
	outfile : gzFile;
	ioerr : integer;
	errorcode : integer;
begin
  AssignFile (infile, FFileSource);
  {$I-}
  Reset (infile,1);
  {$I+}
  ioerr := IOResult;
  if (ioerr <> 0) then begin
	if FWindowOnError then
		 MessageDlg('Can''t open: '+FFileSource, mtError, [mbAbort], 0);
	errorcode := 1
  end
  else begin
	  outmode := 'w  ';
	  s := IntToStr(FCompressionLevel);
	  outmode[2] := s[1];
	  case FCompressionType of
		   Standard    : outmode[3] := ' ';
		   HuffmanOnly : outmode[3] := 'h';
		   Filtered    : outmode[3] := 'f';
	  end;
	  outfile := gzopen (FFileDestination, outmode);

	  if (outfile = NIL) then begin
		 if FWindowOnError then
			  MessageDlg('Can''t open: '+FFileDestination, mtError, [mbAbort], 0);
		 close( infile);
		 errorcode := 2
	  end
	  else begin
		 errorcode := gz_compress(infile, outfile);
		 if errorcode <> 0 then errorcode := errorcode+100
		 else
			if FDeleteSource then erase (infile);
	  end
   end;
   Gzip := errorcode
end;

{ Gzip ---------------------------------------------------------
# Returns 0 - File decompressed
#         1 - Could not open FFileIn
#         2 - Could not create FFileOut
#         3 - FFileIn not a valid gzip-file
---------------------------------------------------------------}
function TGzip.Gunzip : integer;
var //len : integer;
	infile : gzFile;
	outfile, f : file;
	ioerr : integer;
	errorcode : integer;
	fsize : longword;
begin
  errorcode := 0;

  AssignFile( f, FFileDestination);
  {$I-}
  Reset( f, 1);
  {$I+}
  ioerr := IOResult;
  if (ioerr <> 0) then begin
	if FWindowOnError then
		 MessageDlg('Can''t open: '+FFileSource, mtError, [mbAbort], 0);
	errorcode := 1
  end
  else begin
	fsize := FileSize( f);
	Close( f);

	infile := gzopen (FFileSource, 'r');
	if (infile = NIL) then begin
		if FWindowOnError then
		   MessageDlg('Can''t open: '+FFileSource, mtError, [mbAbort], 0);
		errorcode := 1
	end
	else begin
		AssignFile (outfile, FFileDestination);
		{$I-}
		Rewrite (outfile,1);
		{$I+}
		ioerr := IOResult;
		if (ioerr <> 0) then begin
			if FWindowOnError then
			   MessageDlg('Can''t create: '+FFileSource, mtError, [mbAbort], 0);
			errorcode := 2
		end
		else begin
			gz_uncompress (infile, outfile, fsize);
			if FDeleteSource then DeleteFile(FFileSource);
		end
	end
  end;
  Gunzip := errorcode
end;

procedure Register;
begin
  RegisterComponents('Samples', [TGzip]);
  RegisterPropertyEditor(TypeInfo(TAboutProperty), TGzip, 'ABOUT', TAboutProperty);
end;

end.
